@@ -3,17 +3,27 @@
 #' Applies the Hobday et al. (2016) marine heat wave definition to an input time
 #' series of temperature along with a daily date vector.
 #'
-#' @importFrom magrittr %>%
-#' @importFrom stats sd
-#' @importFrom plyr .
-#' @param data A data frame with three columns. They are headed \code{doy},
-#' \code{date} and \code{temp.} \code{doy} is the Julian day running from 1
-#' to 366, but modified so that the day-of-year (doy) vector for non-leap-years
-#' runs 1...59 and then 61...366. For leap years the 60th day is February 29.
-#' The \code{date} column is a vector of dates of class \code{Date}, while
-#' \code{temp} is the temperature. Data of the appropriate format are created
-#' by the function \code{\link{make_whole}}, but your own data can be supplied
+#' @importFrom tidyr %>%
+#'
+#' @param data A data frame with three columns. In the default setting (i.e. ommitting
+#' the arguments \code{doy}, \code{x} and \code{y}; see immediately below), the
+#' data set is expected to have the headers \code{doy}, \code{t} and \code{temp}.
+#' \code{doy} is the Julian day running from 1 to 366, but modified so that the
+#' day-of-year (doy) vector for non-leap-years runs 1...59 and then 61...366.
+#' For leap years the 60th day is February 29. The \code{t} column is a vector
+#' of dates of class \code{Date}, while \code{temp} is the measured variable (by
+#' default it is assumed to be temperature). Data of the appropriate format are
+#' created by the function \code{\link{make_whole}}, but your own data can be supplied
 #' if they meet the criteria specified by \code{\link{make_whole}}.
+#' @param doy If a column headed \code{doy} is not available, another column with
+#' Julian dates can be supplied. This argument accepts the name of that column. The
+#' default name is, of course, \code{doy}.
+#' @param x This column is expected to contain a vector of dates as per the
+#' specification of \code{make_whole}. If a column headed \code{t} is present in
+#' the dataframe, this argument may be ommitted; otherwise, specify the name of
+#' the column with dates here.
+#' @param y This is a column containing the measurement variable. If the column
+#' name differs from the default (i.e. \code{temp}), specify the name here.
 #' @param climatology_start The first full year from which the (varying by
 #' day-of-year) seasonal cycle and extremes threshold are calculated (full being
 #' 366 days if leap year, else 365 days). Note that a default value is provided
@@ -54,8 +64,8 @@
 #' heatwaves, meaning that \code{pctile} should be set the same regardless
 #' if one is calculating heatwaves or cold-spells. For example, if one wants
 #' to calculate heatwaves above the 90th percentile threshold
-#' (the default) one sets \code{pctile} = 90. Likewise, if one would like
-#' identify the most intense cold-spells one must also set \code{pctile} = 90,
+#' (the default) one sets \code{pctile = 90}. Likewise, if one would like
+#' identify the most intense cold-spells one must also set \code{pctile = 90},
 #' even though cold spells are in fact simply the coldest extreme events in a
 #' time series, which statistically equate to values below the 10th percentile.
 #'
@@ -111,11 +121,20 @@
 #' \code{event}, which are the climatology and MHW (or MCS) events, respectively.
 #' The climatology contains the full time series of daily temperatures, as well
 #' as the the seasonal climatology, the threshold and various aspects of the
-#' events that were detected:
+#' events that were detected. The software was designed for detecting extreme
+#' thermal events, and the units specified below reflect that intended purpose.
+#' However, the various other kinds of extreme events may be detected according
+#' to the 'marine heat wave' specifications, and if that is the case, the appropriate
+#' units need to be determined by the user.
 #'   \item{doy}{Julian day (day-of-year). For non-leap years it runs 1...59 and
-#'   61...366, while leap years run 1...366.}
-#'   \item{date}{The date of the temperature measurement.}
-#'   \item{temp}{Seawater temperature on the specified date [deg. C].}
+#'   61...366, while leap years run 1...366. This column will be named differently if
+#'   another name was specified to the \code{doy} argument.}
+#'   \item{t}{The date of the temperature measurement. This column will be
+#'   named differently if another name was specified to the \code{x} argument.}
+#'   \item{temp}{If the software was used for the purpose for which it was designed,
+#'   seawater temperature [deg. C] on the specified date will be returned. This
+#'   column will of course be named differently if another kind of measurement was
+#'   specified to the \code{y} argument.}
 #'   \item{seas_clim_year}{Climatological seasonal cycle [deg. C].}
 #'   \item{thresh_clim_year}{Seasonally varying threshold (e.g., 90th
 #'   percentile) [deg. C].}
@@ -160,8 +179,8 @@
 #' value of the threshold (relative to climatology,
 #' i.e., threshold - climatology.)
 #'
-#' Note that \code{rate_onset} and \code{rate_decline} will return NA
-#' when the event begins/ ends on the first/ last day of the time series. This
+#' Note that \code{rate_onset} and \code{rate_decline} will return \code{NA}
+#' when the event begins/ends on the first/last day of the time series. This
 #' may be particularly evident when the function is applied to large gridded
 #' data sets. Although the other metrics do not contain any errors and
 #' provide sensible values, please take this into account in its
@@ -180,14 +199,17 @@
 #' @export
 #'
 #' @examples
-#' t_dat <- make_whole(sst_WA)
-#' res <- detect(t_dat, climatology_start = 1983, climatology_end = 2012)
+#' ts_dat <- make_whole(sst_WA)
+#' res <- detect(ts_dat, climatology_start = 1983, climatology_end = 2012)
 #' # show a portion of the climatology:
 #' res$clim[1:10, ]
 #' # show some of the heat waves:
 #' res$event[1:5, 1:10]
 detect <-
   function(data,
+           doy = doy,
+           x = t,
+           y = temp,
            climatology_start = 1983,
            climatology_end = 2012,
            pctile = 90,
@@ -201,10 +223,18 @@ detect <-
            max_pad_length = 3,
            cold_spells = FALSE
            # verbose = TRUE, # to be implemented
-           ) {
+  ) {
 
-    t_series <- data
-    t_series$temp <- zoo::na.approx(t_series$temp, maxgap = max_pad_length)
+    temp <- NULL
+
+    doy <- eval(substitute(doy), data)
+    ts.x <- eval(substitute(x), data)
+    ts.y <- eval(substitute(y), data)
+    t_series <- tibble::tibble(doy,
+                               ts.x,
+                               ts.y)
+    rm(doy); rm(ts.x); rm(ts.y)
+    t_series$ts.y <- zoo::na.approx(t_series$ts.y, maxgap = max_pad_length)
 
     if (missing(climatology_start))
       stop("Oops! Please provide a complete year for the start of the climatology.")
@@ -213,25 +243,25 @@ detect <-
       stop("Bummer! Please provide a complete year for the end of the climatology.")
 
     clim_start <- paste(climatology_start, "01", "01", sep = "-")
-    if (t_series$date[1] > clim_start)
+    if (t_series$ts.x[1] > clim_start)
       stop(paste("The specified start date precedes the first day of series, which is",
-                 t_series$date[1]))
+                 t_series$ts.x[1]))
 
     clim_end <- paste(climatology_end, "12", "31", sep = "-")
-    if (clim_end > t_series$date[nrow(t_series)])
+    if (clim_end > t_series$ts.x[nrow(t_series)])
       stop(paste("The specified end date follows the last day of series, which is",
-                 t_series$date[nrow(t_series)]))
+                 t_series$ts.x[nrow(t_series)]))
 
     if (cold_spells)
-      t_series$temp <- -t_series$temp
+      t_series$ts.y <- -t_series$ts.y
 
     tDat <- t_series %>%
-      dplyr::filter(date >= clim_start & date <= clim_end) %>%
-      dplyr::mutate(date = lubridate::year(date)) %>%
-      tidyr::spread(date, temp)
+      dplyr::filter(ts.x >= clim_start & ts.x <= clim_end) %>%
+      dplyr::mutate(ts.x = lubridate::year(ts.x)) %>%
+      tidyr::spread(ts.x, ts.y)
 
     all_NA <- apply(tDat[59:61, ], 2, function(x) !all(is.na(x)))
-    no_NA <- names(all_NA[all_NA > 0]) # compatibility with zoo < 1.7.13...
+    no_NA <- names(all_NA[all_NA > 0])
     tDat[59:61, no_NA] <- zoo::na.approx(tDat[59:61, no_NA], maxgap = 1, na.rm = TRUE)
     tDat <- rbind(utils::tail(tDat, window_half_width),
                   tDat, utils::head(tDat, window_half_width))
@@ -252,7 +282,7 @@ detect <-
           names = FALSE
         )
       var_clim_year[i] <-
-        sd(
+        stats::sd(
           c(t(tDat[(i - (window_half_width)):(i + window_half_width), 2:ncol(tDat)])),
           na.rm = TRUE
         )
@@ -268,7 +298,7 @@ detect <-
       )
 
     if (smooth_percentile) {
-      clim %<>%
+      clim <- clim %>%
         dplyr::mutate(
           seas_clim_year = raster::movingFun(
             seas_clim_year,
@@ -301,32 +331,33 @@ detect <-
         )
     }
 
-    if(clim_only) {
+    if (clim_only) {
       t_series <- merge(data, clim, by = "doy")
-      t_series <- t_series[order(t_series$date),]
+      t_series <- t_series[order(t_series$ts.x),]
       return(t_series)
 
     } else {
-      t_series %<>% dplyr::inner_join(clim, by = "doy")
-      t_series$temp[is.na(t_series$temp)] <- t_series$seas_clim_year[is.na(t_series$temp)]
-      t_series$thresh_criterion <- t_series$temp > t_series$thresh_clim_year
+      t_series <- t_series %>%
+        dplyr::inner_join(clim, by = "doy")
+      t_series$ts.y[is.na(t_series$ts.y)] <- t_series$seas_clim_year[is.na(t_series$ts.y)]
+      t_series$thresh_criterion <- t_series$ts.y > t_series$thresh_clim_year
       ex1 <- rle(t_series$thresh_criterion)
       ind1 <- rep(seq_along(ex1$lengths), ex1$lengths)
       s1 <- split(zoo::index(t_series$thresh_criterion), ind1)
       proto_events <- s1[ex1$values == TRUE]
-      index_stop <- index_start <- NULL ###
+      index_stop <- index_start <- NULL
       proto_events_rng <-
         lapply(proto_events, function(x)
           data.frame(index_start = min(x), index_stop = max(x)))
 
-      duration <- NULL ###
+      duration <- NULL
 
       protoFunc <- function(proto_data) {
         out <- proto_data %>%
           dplyr::mutate(duration = index_stop - index_start + 1) %>%
           dplyr::filter(duration >= min_duration) %>%
-          dplyr::mutate(date_start = t_series$date[index_start]) %>%
-          dplyr::mutate(date_stop = t_series$date[index_stop])
+          dplyr::mutate(date_start = t_series$ts.x[index_start]) %>%
+          dplyr::mutate(date_stop = t_series$ts.x[index_stop])
       }
 
       proto_events <- do.call(rbind, proto_events_rng) %>%
@@ -352,9 +383,9 @@ detect <-
         dplyr::mutate(duration = index_stop - index_start + 1)
 
       if (any(proto_gaps$duration >= 1 & proto_gaps$duration <= max_gap)) {
-        proto_gaps %<>%
-          dplyr::mutate(date_start = t_series$date[index_start]) %>%
-          dplyr::mutate(date_stop = t_series$date[index_stop]) %>%
+        proto_gaps <- proto_gaps %>%
+          dplyr::mutate(date_start = t_series$ts.x[index_start]) %>%
+          dplyr::mutate(date_stop = t_series$ts.x[index_stop]) %>%
           dplyr::filter(duration >= 1 & duration <= max_gap)
       } else {
         join_across_gaps <- FALSE
@@ -374,7 +405,7 @@ detect <-
       ind3 <- rep(seq_along(ex3$lengths), ex3$lengths)
       s3 <- split(zoo::index(t_series$event), ind3)
       events <- s3[ex3$values == TRUE]
-      event_no <- NULL ###
+      event_no <- NULL
       events_rng <-
         lapply(events, function(x)
           data.frame(index_start = min(x), index_stop = max(x)))
@@ -389,61 +420,53 @@ detect <-
           rep(i, length = events$duration[i])
       }
 
-      events_list <- plyr::dlply(events, c("event_no"), function(x)
+      int_mean <- int_max <- int_cum <- int_mean_rel_thresh <-
+        int_max_rel_thresh <- int_cum_rel_thresh <- int_mean_abs <-
+        int_max_abs <- int_cum_abs <- int_mean_norm <- int_max_norm <-
+        rate_onset <- rate_decline <- mhw_rel_thresh <-
+        rel_thresh_norm <- mhw_rel_seas <- NULL
+
+      events_list <- plyr::dlply(events, c("event_no"), function(df)
         with(
           t_series,
           data.frame(
-            date = c(date[x$index_start:x$index_stop]),
-            temp = c(temp[x$index_start:x$index_stop]),
-            seas_clim_year = c(seas_clim_year[x$index_start:x$index_stop]),
-            thresh_clim_year = c(thresh_clim_year[x$index_start:x$index_stop]),
-            mhw_rel_seas = c(temp[x$index_start:x$index_stop]) - c(seas_clim_year[x$index_start:x$index_stop]),
-            mhw_rel_thresh = c(temp[x$index_start:x$index_stop]) - c(thresh_clim_year[x$index_start:x$index_stop]),
-            rel_thresh_norm = c(temp[x$index_start:x$index_stop]) - c(thresh_clim_year[x$index_start:x$index_stop]) /
-              c(thresh_clim_year[x$index_start:x$index_stop]) - c(seas_clim_year[x$index_start:x$index_stop])
+            ts.x = c(ts.x[df$index_start:df$index_stop]),
+            ts.y = c(ts.y[df$index_start:df$index_stop]),
+            seas_clim_year = c(seas_clim_year[df$index_start:df$index_stop]),
+            thresh_clim_year = c(thresh_clim_year[df$index_start:df$index_stop]),
+            mhw_rel_seas = c(ts.y[df$index_start:df$index_stop]) - c(seas_clim_year[df$index_start:df$index_stop]),
+            mhw_rel_thresh = c(ts.y[df$index_start:df$index_stop]) - c(thresh_clim_year[df$index_start:df$index_stop]),
+            rel_thresh_norm = c(ts.y[df$index_start:df$index_stop]) - c(thresh_clim_year[df$index_start:df$index_stop]) /
+              c(thresh_clim_year[df$index_start:df$index_stop]) - c(seas_clim_year[df$index_start:df$index_stop])
           )
         )
       )
 
-      int_mean <- int_max <- int_cum <- int_mean_rel_thresh <-
-        int_max_rel_thresh <- int_cum_rel_thresh <- int_mean_abs <-
-        int_max_abs <- int_cum_abs <- int_mean_norm <- int_max_norm <-
-        temp <- doy <- rate_onset <- rate_decline <- NULL ###
-      events$date_peak <-
-        plyr::ldply(events_list, function(x)
-          x$date[x$mhw_rel_seas == max(x$mhw_rel_seas)][1])[, 2]
-      events$int_mean <-
-        plyr::ldply(events_list, function(x) mean(x$mhw_rel_seas))[, 2]
-      events$int_max <-
-        plyr::ldply(events_list, function(x) max(x$mhw_rel_seas))[, 2]
-      events$int_var <-
-        plyr::ldply(events_list, function(x) sqrt(stats::var(x$mhw_rel_seas)))[, 2]
-      events$int_cum <-
-        plyr::ldply(events_list, function(x) max(cumsum(x$mhw_rel_seas)))[, 2]
-      events$int_mean_rel_thresh <-
-        plyr::ldply(events_list, function(x) mean(x$mhw_rel_thresh))[, 2]
-      events$int_max_rel_thresh <-
-        plyr::ldply(events_list, function(x) max(x$mhw_rel_thresh))[, 2]
-      events$int_var_rel_thresh <-
-        plyr::ldply(events_list, function(x) sqrt(stats::var(x$mhw_rel_thresh)))[, 2]
-      events$int_cum_rel_thresh <-
-        plyr::ldply(events_list, function(x) max(cumsum(x$mhw_rel_thresh)))[, 2]
-      events$int_mean_abs <-
-        plyr::ldply(events_list, function(x) mean(x$temp))[, 2]
-      events$int_max_abs <-
-        plyr::ldply(events_list, function(x) max(x$temp))[, 2]
-      events$int_var_abs <-
-        plyr::ldply(events_list, function(x) sqrt(stats::var(x$temp)))[, 2]
-      events$int_cum_abs <-
-        plyr::ldply(events_list, function(x) max(cumsum(x$temp)))[, 2]
-      events$int_mean_norm <-
-        plyr::ldply(events_list, function(x) mean(x$rel_thresh_norm))[, 2]
-      events$int_max_norm <-
-        plyr::ldply(events_list, function(x) max(x$rel_thresh_norm))[, 2]
+      events <- cbind(events,
+                      events_list %>%
+                        dplyr::bind_rows(.id = "event_no") %>%
+                        dplyr::group_by(event_no) %>%
+                        dplyr::summarise(date_peak = ts.x[mhw_rel_seas == max(mhw_rel_seas)][1],
+                                         int_mean = mean(mhw_rel_seas),
+                                         int_max = max(mhw_rel_seas),
+                                         int_var = sqrt(stats::var(mhw_rel_seas)),
+                                         int_cum = max(cumsum(mhw_rel_seas)),
+                                         int_mean_rel_thresh = mean(mhw_rel_thresh),
+                                         int_max_rel_thresh = max(mhw_rel_thresh),
+                                         int_var_rel_thresh = sqrt(stats::var(mhw_rel_thresh)),
+                                         int_cum_rel_thresh = max(cumsum(mhw_rel_thresh)),
+                                         int_mean_abs = mean(ts.y),
+                                         int_max_abs = max(ts.y),
+                                         int_var_abs = sqrt(stats::var(ts.y)),
+                                         int_cum_abs = max(cumsum(ts.y)),
+                                         int_mean_norm = mean(rel_thresh_norm),
+                                         int_max_norm = max(rel_thresh_norm)) %>%
+                        dplyr::arrange(as.numeric(event_no)) %>%
+                        dplyr::select(-event_no))
 
-      mhw_rel_seas <- t_series$temp - t_series$seas_clim_year
+      mhw_rel_seas <- t_series$ts.y - t_series$seas_clim_year
       A <- mhw_rel_seas[events$index_start]
-      B <- t_series$temp[events$index_start - 1]
+      B <- t_series$ts.y[events$index_start - 1]
       C <- t_series$seas_clim_year[events$index_start - 1]
       if (length(B) + 1 == length(A)) {
         B <- c(NA, B)
@@ -453,13 +476,13 @@ detect <-
 
       events$rate_onset <- ifelse(
         events$index_start > 1,
-        (events$int_max - mhw_rel_seas_start) / (as.numeric( ## RWS: The other part of the calculation
+        (events$int_max - mhw_rel_seas_start) / (as.numeric(
           difftime(events$date_peak, events$date_start, units = "days")) + 0.5),
         NA
-        )
+      )
 
       D <- mhw_rel_seas[events$index_stop]
-      E <- t_series$temp[events$index_stop + 1]
+      E <- t_series$ts.y[events$index_stop + 1]
       F <- t_series$seas_clim_year[events$index_stop + 1]
       mhw_rel_seas_end <- 0.5 * (D + E - F)
 
@@ -487,13 +510,17 @@ detect <-
           rate_decline = -rate_decline
         )
         t_series <- t_series %>% dplyr::mutate(
-          temp = -temp,
+          ts.y = -ts.y,
           seas_clim_year = -seas_clim_year,
           thresh_clim_year = -thresh_clim_year
         )
       }
 
-      list(clim = dplyr::group_by(t_series),
-           event = dplyr::group_by(events))
+      names(t_series)[1] <- paste(substitute(doy))
+      names(t_series)[2] <- paste(substitute(x))
+      names(t_series)[3] <- paste(substitute(y))
+
+      list(clim = t_series,
+           event = events)
     }
   }
